/*
 *  @Copyright (C) 2008 Kim Jin Chul (shriekout@gmail.com).
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gst/gst.h>
#include <panel-applet.h>
#include "support.h"
#include "gdic_sh.h"
#include "interface.h"

static GMainLoop *loop;
static gboolean text_view_shown = FALSE;

gboolean delete_event(GtkWidget *widget,
						GdkEvent *event,
						gpointer data)
{
	return FALSE;
}

void destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

void on_input_entry_activate(GtkWidget *widget, gpointer data)
{
	gchar *buf;

	text_view_shown = TRUE;

	gtk_widget_show(dic_w);

	buf = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);

	read_content(buf);

	g_free(buf);
}

void on_find_button_clicked(GtkWidget *widget, gpointer data)
{
	on_input_entry_activate(GTK_WIDGET(data), NULL);
}


static gboolean bus_call(GstBus *bus,
							GstMessage *msg,
							gpointer user_data)
{
	switch (GST_MESSAGE_TYPE(msg))
	{
		case GST_MESSAGE_EOS: {
/*			g_message("End-of-stream");*/
			g_main_loop_quit(loop);
			break;
		}
		case GST_MESSAGE_ERROR: {
			gchar *debug;
			GError *err;

			gst_message_parse_error(msg, &err, &debug);
			g_free(debug);

			g_error("%s", err->message);
			g_error_free(err);

			g_main_loop_quit(loop);
			break;
		}
		default:
			break;
	}

	return TRUE;
}
void on_sound_event_box_button_press(GtkWidget *widget, GdkEventButton *event, gpointer uri)
{
	GstElement *pipeline;

	loop = g_main_loop_new(NULL, FALSE);

	pipeline = gst_element_factory_make("playbin", "player");

	g_object_set(G_OBJECT(pipeline), "uri", uri, NULL);

	{
		GstBus *bus;

		bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));
		gst_bus_add_watch(bus, bus_call, NULL);
		gst_object_unref(bus);
	}

	gst_element_set_state(pipeline, GST_STATE_PLAYING);

	g_main_loop_run(loop);

	gst_element_set_state(pipeline, GST_STATE_NULL);
	gst_object_unref(GST_OBJECT(pipeline));
}

void on_entry_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	panel_applet_request_focus(PANEL_APPLET(data), event->time);
}

void on_close_button_clicked(GtkWidget *widget, gpointer data)
{
	if (text_view_shown == TRUE)
		gtk_widget_hide(dic_w);
	else
		gtk_widget_show(dic_w);

	text_view_shown = !text_view_shown;
}

void on_about_activate(BonoboUIComponent *uic, gpointer data)
{
		display_about_dialog();
}

gboolean on_applet_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if (event->button != 1) {
		g_signal_stop_emission_by_name (widget, "button-press-event");
		return FALSE;
	} else {
		on_close_button_clicked(NULL, NULL);
		return TRUE;
	}
}

void appletChangeBackground(PanelApplet *applet,
								PanelAppletBackgroundType type,
								GdkColor *color,
								GdkPixmap *pixmap)
{
	GtkRcStyle *rc_style;
	GtkStyle *style;

	/* reset style */
	gtk_widget_set_style(GTK_WIDGET(applet), NULL);
	rc_style = gtk_rc_style_new();
	gtk_widget_modify_style(GTK_WIDGET(applet), rc_style);
	gtk_rc_style_unref(rc_style);

	switch (type) {
		case PANEL_NO_BACKGROUND:
			break;
		case PANEL_COLOR_BACKGROUND:
			gtk_widget_modify_bg(GTK_WIDGET(applet), GTK_STATE_NORMAL, color);
			break;
		case PANEL_PIXMAP_BACKGROUND:
			style = gtk_style_copy(GTK_WIDGET(applet)->style);

			if (style->bg_pixmap[GTK_STATE_NORMAL])
				g_object_unref(style->bg_pixmap[GTK_STATE_NORMAL]);

			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref(pixmap);
			gtk_widget_set_style(GTK_WIDGET(applet), style);
			g_object_unref(style);
			break;
		default:
			break;
	}
}
