/*
 *  @Copyright (C) 2008 Kim Jin Chul (shriekout@gmail.com).
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <string.h>
#include <panel-applet.h>
#include "callback.h"
#include "gdic_sh.h"
#include "interface.h"

static GtkWidget *create_menubar(void)
{
	GtkWidget *menubar;
	GtkWidget *menuitem;
	GtkWidget *submenu;
	GtkWidget *item;
	GtkWidget *close_menu;
	GtkWidget *about_menu;

	menubar = gtk_menu_bar_new();

	menuitem = gtk_menu_item_new_with_label("파일");
	gtk_container_add(GTK_CONTAINER(menubar), menuitem);
	gtk_widget_show(menuitem);

	submenu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);

	item = gtk_separator_menu_item_new();
	gtk_container_add(GTK_CONTAINER(submenu), item);
	gtk_widget_set_sensitive(item, FALSE);
	gtk_widget_show(item);

	close_menu = gtk_image_menu_item_new_from_stock("gtk-close", NULL);
	gtk_container_add(GTK_CONTAINER(submenu), close_menu);
	gtk_widget_show(close_menu);

	menuitem = gtk_menu_item_new_with_label("도움말");
	gtk_container_add(GTK_CONTAINER(menubar), menuitem);
	gtk_widget_show(menuitem);

	submenu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);

	about_menu = gtk_image_menu_item_new_from_stock("gtk-about", NULL);
	gtk_container_add(GTK_CONTAINER(submenu), about_menu);
	gtk_widget_show(about_menu);

	g_signal_connect(G_OBJECT(close_menu), "activate",
						G_CALLBACK(destroy), NULL);
	g_signal_connect(G_OBJECT(about_menu), "activate",
						G_CALLBACK(display_about_dialog), NULL);

	return menubar;
}

static GtkWidget *create_entry_box(void)
{
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *input_entry;
	GtkWidget *find_button;

	hbox = gtk_hbox_new(FALSE, 5);

	label = gtk_label_new("찾을 단어");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	input_entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), input_entry, TRUE, TRUE, 0);
	gtk_widget_show(input_entry);
	g_signal_connect(G_OBJECT(input_entry), "activate",
						G_CALLBACK(on_input_entry_activate), NULL);
	
	find_button = gtk_button_new_from_stock("gtk-find");
	gtk_box_pack_start(GTK_BOX(hbox), find_button, FALSE, FALSE, 0);
	gtk_widget_show(find_button);
	g_signal_connect(G_OBJECT(find_button), "clicked",
						G_CALLBACK(on_find_button_clicked), (gpointer)input_entry);

	return hbox;
}

GtkWidget *create_dictionary_window(const char *kind)
{
	GtkWidget *window;
	GdkPixbuf *pixbuf;
	GtkWidget *vbox;
	GtkWidget *menubar;
	GtkWidget *hbox;
	GtkWidget *scrolledwindow;
	GtkTextBuffer *content_buffer;
	GtkTextTag *normal_tag;
	GtkTextTag *word_tag;
	GtkTextTag *sp_tag;
	GtkTextTag *origin_tag;
	GtkTextTag *error_tag;
	GtkWidget *content_view;
	GtkWidget *close_button;
	GtkWidget *statusbar;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	gtk_window_set_title(GTK_WINDOW(window), "gdic_sh");
	pixbuf = gdk_pixbuf_new_from_file(GDIC_SH_ICON, NULL);
	gtk_window_set_icon(GTK_WINDOW(window), pixbuf);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
	g_object_unref(pixbuf);

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_widget_show(vbox);

	if (!strcmp(kind, "APPLICATION")) {
		gtk_window_set_default_size(GTK_WINDOW(window), 400, 400);

		menubar = create_menubar();
		gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
		gtk_widget_show(menubar);

		hbox = create_entry_box();
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
		gtk_widget_show(hbox);
	} else if (!strcmp(kind, "APPLET")) {
		gtk_window_set_default_size(GTK_WINDOW(window), 400, 200);
		gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
		gtk_window_stick(GTK_WINDOW(window));
		gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), TRUE);
	}

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_SHADOW_IN);
	gtk_widget_show(scrolledwindow);

	content_buffer = gtk_text_buffer_new(NULL);
	normal_tag = gtk_text_buffer_create_tag(content_buffer, NULL,
													"font", "12",
													NULL);
	word_tag = gtk_text_buffer_create_tag(content_buffer, NULL,
												"font", "bold 15",
												NULL);
	sp_tag = gtk_text_buffer_create_tag(content_buffer, NULL,
												"foreground", "blue",
												"font", "10",
												NULL);
	origin_tag = gtk_text_buffer_create_tag(content_buffer, NULL,
												"foreground", "green",
												"font", "10",
												NULL);
	error_tag = gtk_text_buffer_create_tag(content_buffer, NULL,
									"foreground", "red",
									"font", "bold 15",
									NULL);
	g_object_set_data(G_OBJECT(content_buffer), "normal_tag", normal_tag);
	g_object_set_data(G_OBJECT(content_buffer), "word_tag", word_tag);
	g_object_set_data(G_OBJECT(content_buffer), "sp_tag", sp_tag);
	g_object_set_data(G_OBJECT(content_buffer), "origin_tag", origin_tag);
	g_object_set_data(G_OBJECT(content_buffer), "error_tag", error_tag);
	g_object_set_data(G_OBJECT(window), "content_buffer", content_buffer);

	content_view = gtk_text_view_new_with_buffer(content_buffer);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(content_view), GTK_WRAP_CHAR);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(content_view), FALSE);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), content_view);
	gtk_container_set_border_width(GTK_CONTAINER(content_view), 3);
	gtk_widget_show(content_view);
	g_object_set_data(G_OBJECT(window), "content_view", content_view);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	close_button = gtk_button_new_from_stock("gtk-close");
	gtk_box_pack_end(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
	gtk_widget_show(close_button);

	statusbar = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(window), "statusbar", statusbar);
	gtk_widget_show(statusbar);

	g_signal_connect(G_OBJECT(window), "delete_event",
						G_CALLBACK(delete_event), NULL);
	g_signal_connect(G_OBJECT(window), "destroy",
						G_CALLBACK(destroy), NULL);

	if (!strcmp(kind, "APPLICATION")) {
		g_signal_connect(G_OBJECT(close_button), "clicked",
							G_CALLBACK(destroy), NULL);
	} else if (!strcmp(kind, "APPLET")) {
		g_signal_connect(G_OBJECT(close_button), "clicked",
							G_CALLBACK(on_close_button_clicked), NULL);
	}

	return window;
}

void display_about_dialog(void)
{
	GdkPixbuf *logo;
	GError *error = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {"김진철 (shriekout@gmail.com)", NULL};
	const gchar *license = "이 프로그램은 자유 소프트웨어 입니다. 이 소프트웨어의 피양도자는 자유 소프트 재단이 공표한 GNU 일반 공중 사용 허가서 2판 또는 그  이후 판을 임의로 선택해서, 그 규정에 따라 프로그램을 개작하거나 재배포할 수 있습니다.\n\n이 프로그램은 유용하게 사용될 수 있으리라는 희망에서 배포되고 있지만, 특정한 목적에 맞는 적합성 여부나 판매용으로 사용할 수 있으리라는 묵시적인 보증을 포함한 어떠한 형태의 보증도 제공하지 않습니다. 보다 자세한 사항에 대해서는 GNU 일반 공중 사용 허가서를 참고하시기 바랍니다.\n\nGNU 일반 공증 사용 허가서는 이 프로그램과 함께 제공됩니다. 만약, 이 문서가 누락되어 있다면 자유 소프트웨어 재단으로 문의하시기 바랍니다. (자유 소프트웨어 재단:Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA)";

	if (about != NULL) {
		gtk_window_present(GTK_WINDOW(about));
		return ;
	}

	logo = gdk_pixbuf_new_from_file(GDIC_SH_ICON, &error);

	about = gtk_about_dialog_new();
	gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), "gdic_sh");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), "Copyright (C) 2007- 김진철");
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), "이 프로그램은 네이버(http://www.naver.com) 영한(한영)사전에서 영어단어 뜻을 찾아 보여줍니다.");
/*	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), website);
	gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about), website_label);*/
	gtk_about_dialog_set_wrap_license(GTK_ABOUT_DIALOG(about), TRUE);
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), license);
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), authors);
	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(about), authors);
	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about), logo);
	gdk_pixbuf_unref(logo);

	gtk_window_set_icon_from_file(GTK_WINDOW(about), GDIC_SH_ICON, NULL);

	gtk_dialog_run(GTK_DIALOG(about));
	gtk_widget_destroy(about);
	about = NULL;
}

GtkWidget *create_applet_box(PanelApplet *applet)
{
	GtkWidget *hbox;
	GtkWidget *image;
	GtkWidget *applet_button;
	GtkWidget *entry;

	hbox = gtk_hbox_new(FALSE, 0);
	applet_button = gtk_button_new();
	gtk_box_pack_start(GTK_BOX(hbox), applet_button, FALSE, FALSE, 0);
	gtk_widget_show(applet_button);
	g_signal_connect(G_OBJECT(applet_button), "button_press_event",
						G_CALLBACK (on_applet_button_press_event),
						NULL);

	image = gtk_image_new_from_file(APPLET_ICON);
	gtk_button_set_image(GTK_BUTTON(applet_button), image);
	gtk_widget_show(image);

	entry = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY(entry), 12);
	gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
	gtk_widget_show(entry);
	g_signal_connect (entry, "button-press-event",
					    G_CALLBACK (on_entry_button_press_event),
				    	(gpointer)applet);
	g_signal_connect(G_OBJECT(entry), "activate",
						G_CALLBACK(on_input_entry_activate), NULL);
	g_signal_connect(G_OBJECT(applet), "change-background",
						G_CALLBACK(appletChangeBackground), NULL);


	return hbox;
}
